/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: Host.cc,v 1.15 2002/06/11 21:20:24 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifdef __MINGW32__
#include <assert.h>
#endif

#include <fwbuilder/libfwbuilder-config.h>

#include <fwbuilder/Host.hh>
#include <fwbuilder/Interface.hh>
#include <fwbuilder/Management.hh>

#include <fwbuilder/XMLTools.hh>

using namespace std;
using namespace libfwbuilder;

const char *Host::TYPENAME={"Host"};

Host::Host()  
{
    setStr("address", "");
}

Host::Host(const FWObject *root) : Address(root)
{
    setStr("address", "");
    add(new HostOptions() );
    setDefaults();
}

Host::~Host() 
{
  clearChildren();
}

void Host::fromXML(xmlNodePtr root) throw(FWException)
{
    const char *n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("address")));
    assert(n!=NULL);
    setStr("address", n);

    FWObject::fromXML(root);
}

xmlNodePtr Host::toXML(xmlNodePtr parent) throw(FWException)
{
    xmlNodePtr me = FWObject::toXML(parent, false);
    FWObject *o;

    for(FWObjectTypedChildIterator j=findByType(Interface::TYPENAME); j!=j.end(); ++j)
 	if((o=(*j))!=NULL )
 	    o->toXML(me);
    
    o=getFirstByType( Management::TYPENAME );
    if (o) o->toXML(me);

    o=getFirstByType( HostOptions::TYPENAME );
    if (o) o->toXML(me);

    return me;
}


bool  Host::validateChild(FWObject *o)
{ 
    string otype=o->getTypeName();
    return (FWObject::validateChild(o) && 
	    (otype==Interface::TYPENAME || 
	     otype==Management::TYPENAME ||
	     otype==HostOptions::TYPENAME) );
}


/*
 *  TODO:    Implement syntax checks for interfaces
 *
 */
bool Host::Appropriate(Interface *i)
{
  return true;
}

void Host::addInterface(Interface *i)
{
    add(i);
}

void Host::removeInterface(Interface *i)
{
    remove(i);
}

FWOptions* Host::getOptionsObject()
{
    return FWOptions::cast( getFirstByType(HostOptions::TYPENAME) );
}

void Host::setDefaults()
{
    FWOptions *fopt=getOptionsObject();

    if (!fopt->exists("use_mac_addr"))
	fopt->setBool("use_mac_addr" , false);

}

IPAddress Host::getAddress() const
{
    return IPAddress( getStr("address") );
}

Netmask   Host::getNetmask() const
{
    return Netmask("255.255.255.255");
}

Management *Host::getManagementObject()
{
    Management *res = dynamic_cast<Management *>(getFirstByType(Management::TYPENAME));
    if(!res)
        add(res = new Management());
    return res;
}
