/*

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: NATCompiler.cc,v 1.18 2002/05/24 02:08:02 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/

#ifdef __MINGW32__
#include <assert.h>
#endif

#include "NATCompiler.hh"

#include "fwbuilder/NAT.hh"
#include "fwbuilder/Rule.hh"
#include "fwbuilder/InterfacePolicy.hh"
#include "fwbuilder/Firewall.hh"
#include "fwbuilder/RuleSet.hh"
#include "fwbuilder/IPAddress.hh"
#include "fwbuilder/Interface.hh"
#include "fwbuilder/FWObjectDatabase.hh"
#include "fwbuilder/RuleElement.hh"

#include <iostream>
#include <iomanip>
#include <sstream>

using namespace fwcompiler;
using namespace libfwbuilder;
using namespace std;


int NATCompiler::prolog()
{
    Compiler::prolog();

    FWObject  *nat=fw->getFirstByType(NAT::TYPENAME);
    assert(nat);

    combined_ruleset = new NAT();
    fw->add( combined_ruleset );

    temp_ruleset = new NAT();   // working copy of the policy
    fw->add( temp_ruleset );


/*
 *  build combined policy by collapsing all the rules together.
 *  store ID of the interface in each rule of interface policy.
 *
 *  also calculate global numbers for all rules and store them, too.
 *  These are used to detect rule shading.
 */
    int global_num=0;

	
    list<FWObject*> l3=nat->getByType(NATRule::TYPENAME);
    for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) {
	Rule *r= Rule::cast(*j);
	if (r->isDisabled()) continue;
	r->setStr("interface_id","");
	r->setStr("rule_label", createRuleLabel("NAT", r->getPosition()) );
	r->setInt("rule_number",global_num); global_num++;
	combined_ruleset->add( r );
    }

    initialized=true;

    return combined_ruleset->size();
}


bool NATCompiler::ExpandMultipleAddresses::processNext()
{
    NATRule *rule=getNext(); if (rule==NULL) return false;

    tmp_queue.push_back(rule);

    RuleElement *rel;
    if (rule->getRuleType()==NATRule::SNAT) {
	rel=rule->getOSrc();    assert(rel);
	compiler->_expandAddr(rule,rel);
	rel=rule->getODst();    assert(rel);
	compiler->_expandAddr(rule,rel);
	rel=rule->getTDst();    assert(rel);
	compiler->_expandAddr(rule,rel);
    }
    if (rule->getRuleType()==NATRule::DNAT) {
	rel=rule->getOSrc();    assert(rel);
	compiler->_expandAddr(rule,rel);
	rel=rule->getTSrc();    assert(rel);
	compiler->_expandAddr(rule,rel);
	rel=rule->getTDst();    assert(rel);
	compiler->_expandAddr(rule,rel);
    }
    if (rule->getRuleType()==NATRule::Redirect) {
	rel=rule->getOSrc();    assert(rel);
	compiler->_expandAddr(rule,rel);
	rel=rule->getODst();    assert(rel);
	compiler->_expandAddr(rule,rel);
	rel=rule->getTSrc();    assert(rel);
	compiler->_expandAddr(rule,rel);
    }
    return true;
}

bool NATCompiler::addressRanges::processNext()
{
    NATRule *rule=getNext(); if (rule==NULL) return false;

    tmp_queue.push_back(rule);

    RuleElement *rel;

    rel=rule->getOSrc();    assert(rel);
    compiler->_expandAddressRanges(rule,rel);
    rel=rule->getODst();    assert(rel);
    compiler->_expandAddressRanges(rule,rel);
    rel=rule->getTSrc();    assert(rel);
    compiler->_expandAddressRanges(rule,rel);
    rel=rule->getTDst();    assert(rel);
    compiler->_expandAddressRanges(rule,rel);

    return true;
}




bool NATCompiler::ExpandGroups::processNext()
{
    NATRule *rule=getNext(); if (rule==NULL) return false;

    tmp_queue.push_back(rule);

    RuleElement *osrc=rule->getOSrc();    assert(osrc);
    RuleElement *odst=rule->getODst();    assert(odst);
    RuleElement *osrv=rule->getOSrv();    assert(osrv);
    RuleElement *tsrc=rule->getTSrc();    assert(tsrc);
    RuleElement *tdst=rule->getTDst();    assert(tdst);
    RuleElement *tsrv=rule->getTSrv();    assert(tsrv);

    compiler->expandGroupsInRuleElement(osrc);
    compiler->expandGroupsInRuleElement(odst);
    compiler->expandGroupsInRuleElement(osrv);
    compiler->expandGroupsInRuleElement(tsrc);
    compiler->expandGroupsInRuleElement(tdst);
    compiler->expandGroupsInRuleElement(tsrv);

    return true;
}

bool NATCompiler::ConvertToAtomicForAddresses::processNext()
{
    NATRule *rule=getNext(); if (rule==NULL) return false;

    RuleElementOSrc *osrc=rule->getOSrc();    assert(osrc);
    RuleElementODst *odst=rule->getODst();    assert(odst);
    RuleElementOSrv *osrv=rule->getOSrv();    assert(osrv);

    RuleElementTSrc *tsrc=rule->getTSrc();    assert(tsrc);
    RuleElementTDst *tdst=rule->getTDst();    assert(tdst);
    RuleElementTSrv *tsrv=rule->getTSrv();    assert(tsrv);

    for (FWObject::iterator i1=osrc->begin(); i1!=osrc->end(); ++i1) {
	for (FWObject::iterator i2=odst->begin(); i2!=odst->end(); ++i2) {

	    for (FWObject::iterator i4=tsrc->begin(); i4!=tsrc->end(); ++i4) {
		for (FWObject::iterator i5=tdst->begin(); i5!=tdst->end(); ++i5) {

		    NATRule *r = NATRule::cast(
			compiler->dbcopy->create(NATRule::TYPENAME,false) );
		    r->duplicate(rule);
                    compiler->temp_ruleset->add(r);

		    FWObject *s;

		    s=r->getOSrc();	assert(s);
		    s->clearChildren();
		    s->add( *i1 );

		    s=r->getODst();	assert(s);
		    s->clearChildren();
		    s->add( *i2 );

		    s=r->getOSrv();	assert(s);
		    *s=*osrv;

		    s=r->getTSrc();	assert(s);
		    s->clearChildren();
		    s->add( *i4 );

		    s=r->getTDst();	assert(s);
		    s->clearChildren();
		    s->add( *i5 );

		    s=r->getTSrv();	assert(s);
		    *s=*tsrv;

		    tmp_queue.push_back(r);
		}
	    }
	}
    }

    return true;
}

bool NATCompiler::ConvertToAtomic::processNext()
{
    NATRule *rule=getNext(); if (rule==NULL) return false;

    RuleElementOSrc *osrc=rule->getOSrc();    assert(osrc);
    RuleElementODst *odst=rule->getODst();    assert(odst);
    RuleElementOSrv *osrv=rule->getOSrv();    assert(osrv);

    RuleElementTSrc *tsrc=rule->getTSrc();    assert(tsrc);
    RuleElementTDst *tdst=rule->getTDst();    assert(tdst);
    RuleElementTSrv *tsrv=rule->getTSrv();    assert(tsrv);

    for (FWObject::iterator i1=osrc->begin(); i1!=osrc->end(); ++i1) {
	for (FWObject::iterator i2=odst->begin(); i2!=odst->end(); ++i2) {
	    for (FWObject::iterator i3=osrv->begin(); i3!=osrv->end(); ++i3) {

		for (FWObject::iterator i4=tsrc->begin(); i4!=tsrc->end(); ++i4) {
		    for (FWObject::iterator i5=tdst->begin(); i5!=tdst->end(); ++i5) {
			for (FWObject::iterator i6=tsrv->begin(); i6!=tsrv->end(); ++i6) {

			    NATRule *r = NATRule::cast(
				compiler->dbcopy->create(NATRule::TYPENAME,false) );
			    r->duplicate(rule);
                            compiler->temp_ruleset->add(r);

			    FWObject *s;

			    s=r->getOSrc();	assert(s);
			    s->clearChildren();
			    s->add( *i1 );

			    s=r->getODst();	assert(s);
			    s->clearChildren();
			    s->add( *i2 );

			    s=r->getOSrv();	assert(s);
			    s->clearChildren();
			    s->add( *i3 );


			    s=r->getTSrc();	assert(s);
			    s->clearChildren();
			    s->add( *i4 );

			    s=r->getTDst();	assert(s);
			    s->clearChildren();
			    s->add( *i5 );

			    s=r->getTSrv();	assert(s);
			    s->clearChildren();
			    s->add( *i6 );

			    tmp_queue.push_back(r);

			}
		    }
		}
	    }
	}
    }
    return true;
}


string NATCompiler::debugPrintRule(libfwbuilder::Rule *r)
{
    NATRule *rule=NATRule::cast(r);

    RuleElementOSrc *osrcrel=rule->getOSrc();
    RuleElementODst *odstrel=rule->getODst();
    RuleElementOSrv *osrvrel=rule->getOSrv();

    RuleElementTSrc *tsrcrel=rule->getTSrc();
    RuleElementTDst *tdstrel=rule->getTDst();
    RuleElementTSrv *tsrvrel=rule->getTSrv();


    ostringstream str;

//    str << setw(70) << setfill('-') << "-";

    int no=0;
    FWObject::iterator i1=osrcrel->begin();
    FWObject::iterator i2=odstrel->begin(); 
    FWObject::iterator i3=osrvrel->begin();

    FWObject::iterator i4=tsrcrel->begin();
    FWObject::iterator i5=tdstrel->begin(); 
    FWObject::iterator i6=tsrvrel->begin();

    while ( i1!=osrcrel->end() || i2!=odstrel->end() || i3!=osrvrel->end() ||
            i4!=tsrcrel->end() || i5!=tdstrel->end() || i6!=tsrvrel->end() ) {

        str  << endl;

        string osrc=" ";
        string odst=" ";
        string osrv=" ";

        string tsrc=" ";
        string tdst=" ";
        string tsrv=" ";

        if (i1!=osrcrel->end()) {
            FWObject *o=*i1;
            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
            osrc=o->getName();
        }

        if (i2!=odstrel->end()) {
            FWObject *o=*i2;
            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
            odst=o->getName();
        }

        if (i3!=osrvrel->end()) {
            FWObject *o=*i3;
            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
            osrv=o->getName();
        }

        if (i4!=tsrcrel->end()) {
            FWObject *o=*i4;
            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
            tsrc=o->getName();
        }

        if (i5!=tdstrel->end()) {
            FWObject *o=*i5;
            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
            tdst=o->getName();
        }

        if (i6!=tsrvrel->end()) {
            FWObject *o=*i6;
            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
            tsrv=o->getName();
        }

        int w=0;
        if (no==0) {
            str << rule->getStr("rule_label");
            w=rule->getStr("rule_label").length();
        }
        
        str <<  setw(8-w)  << setfill(' ') << " ";

        str <<  setw(16) << setfill(' ') << osrc.c_str();
        str <<  setw(16) << setfill(' ') << odst.c_str();
        str <<  setw(10) << setfill(' ') << osrv.c_str();
//        str <<  endl;
//
//        str <<  setw(8)  << setfill(' ') << " ";
        str <<  setw(16) << setfill(' ') << tsrc.c_str();
        str <<  setw(16) << setfill(' ') << tdst.c_str();
        str <<  setw(10) << setfill(' ') << tsrv.c_str();


        ++no;

        if ( i1!=osrcrel->end() ) ++i1;
        if ( i2!=odstrel->end() ) ++i2;
        if ( i3!=osrvrel->end() ) ++i3;
        if ( i4!=tsrcrel->end() ) ++i4;
        if ( i5!=tdstrel->end() ) ++i5;
        if ( i6!=tsrvrel->end() ) ++i6;
    }
    return str.str();

}

